//==============================================================================
//  EventGeneratorSettings.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1 
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2013-05-29 16:26:19 -0400 (Wed, 29 May 2013) $
//  $Author: thomas.ullrich@bnl.gov $
//==============================================================================
//        
//  Singleton class.        
//        
//  Notes on verbose settings:       
//  verbose = false:   most messages are suppressed, only severe errors are        
//                     printed. That is the default mode.       
//  verbose = true:    amount of print out depends on the verbose level.       
//    verboseLevel: 1  default - useful for production mode, additional info                        
//                  2  more print-out, information of internal processes,       
//                     still OK for production runs.       
//                  3  even more print-out, OK for test runs but not production       
//                 10  only for debugging - massive output for every event       
//===============================================================================         
#ifndef EventGeneratorSettings_h         
#define EventGeneratorSettings_h         
#include "Settings.h"         
#include "Enumerations.h"         
#include "TLorentzVector.h"         
         
using namespace std;         
         
class EventGeneratorSettings : public Settings {         
public:         
    static EventGeneratorSettings* instance();         
         
    void setVerbose(bool);         
    bool verbose() const;         
         
    void setVerboseLevel(int);         
    int verboseLevel() const;         
             
    void setNumberOfEvents(unsigned long);         
    unsigned long numberOfEvents() const;         
         
    void setTimesToShow(unsigned int);         
    unsigned int timesToShow() const;         
                 
    void setQ2min(double);         
    double Q2min() const;         
    double Qmin() const;         
             
    void setQ2max(double);         
    double Q2max() const;         
    double Qmax() const;         
             
    void setWmin(double);         
    void setW2min(double);         
    double Wmin() const;         
    double W2min() const;         
             
    void setWmax(double);         
    void setW2max(double);         
    double Wmax() const;         
    double W2max() const;         
                 
    int vectorMesonId() const;         
    void setVectorMesonId(int);         
         
    double electronBeamEnergy() const;         
    void setElectronBeamEnergy(double);         
         
    double hadronBeamEnergy() const;         
    void setHadronBeamEnergy(double);         
         
    TLorentzVector eBeam() const;         
    TLorentzVector hBeam() const;         
             
    string          dipoleModelName() const;         
    DipoleModelType dipoleModelType() const;         
    void setDipoleModelType(DipoleModelType);         
         
    string rootfile() const;         
    void setRootfile(const char*);         
         
    unsigned int A() const;         
    void setA(unsigned int);         
         
    bool correctForRealAmplitude() const;         
    void setCorrectForRealAmplitude(bool);         
         
    bool correctSkewedness() const;         
    void setCorrectSkewedness(bool);         
         
    bool enableNuclearBreakup() const;         
    void setEnableNuclearBreakup(bool);         
         
    double maxNuclearExcitationEnergy() const;         
    void setMaxNuclearExcitationEnergy(double);         
  
    double maxLambdaUsedInCorrections() const;         
    void setMaxLambdaUsedInCorrections(double);         

    bool applyPhotonFlux() const;         
    void setApplyPhotonFlux(bool);         

    void setUPC(bool);
    bool UPC() const;

    void setUPCA(unsigned int);
    unsigned int UPCA() const;



private:         
    EventGeneratorSettings();         
    void consolidateSettings();         
             
private:         
    static EventGeneratorSettings* mInstance;         
         
private:         
    bool mVerbose;         
    int  mVerboseLevel;         
         
    unsigned long mNumberOfEvents;         
    unsigned int mTimesToShow;         
    unsigned int mA;         
         
    double mQ2min;         
    double mQ2max;         
    double mWmin;         
    double mWmax;         
             
    int    mVectorMesonId;             
    double mElectronBeamEnergy;         
    double mHadronBeamEnergy;         
    string mRootfile;         
             
    string          mDipoleModelName;         
    DipoleModelType mDipoleModelType;         
         
    bool   mCorrectForRealAmplitude;         
    bool   mCorrectSkewedness;         
    bool   mEnableNuclearBreakup; 
    bool   mApplyPhotonFlux;
      
    double mMaxLambdaUsedInCorrections;         
    double mMaxNuclearExcitationEnergy;         

    bool mUPC;
    unsigned int mUPCA;

};         
         
#endif         
