//==============================================================================
//  Kinematics.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1 
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2013-05-29 16:26:19 -0400 (Wed, 29 May 2013) $
//  $Author: thomas.ullrich@bnl.gov $
//==============================================================================
#ifndef Kinematics_h         
#define Kinematics_h         
#include "Constants.h"         
#include "Settings.h"         
#include "TLorentzVector.h"         
         
class Kinematics {         
public:         
    static double         xprobe(double Q2, double W2, double vmM);         
    static TLorentzVector electronBeam(double eE);         
    static TLorentzVector hadronBeam(double eH);         
    static double         s(const TLorentzVector& e, const TLorentzVector& p);         
    static double         s(double eE, double hE);         
    static double         x(double Q2, double W2);             
    static double         y(double Q2, double x, double s);             
    static double         ymin(double s, double vmM);             
    static double         ymax(double s, double vmM);             
    static double         W2(double Q2, double x);         
    static double         W2(double Q2, double xprobe, double vmM);         
    static double         W2min(double vmM);         
    static double         W2max(double s);         
    static double         Q2min(double y);         
    static double         Q2max(double s);         
    static double         xpomeron(double t, double Q2, double W2, double vmM);         
    static double         tmax(double xpom); // often referred to as tmin implying min |T|         
    static double         tmax(double t, double Q2, double W2, double vmM);         
    static double         tmin(double hE);   // the smallest t value possible          
         
    static bool           valid(double s, double t, double Q2, double W2, double vmMass,          
                                bool useTrueXp = false,          
                                bool verbose = false);         
             
    static bool           error();         
             
private:         
    static bool mError;         
};         
         
#endif         
