//==============================================================================
//  ModeFinderFunctor.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1 
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2014-04-17 13:09:08 -0400 (Thu, 17 Apr 2014) $
//  $Author: thomas.ullrich@bnl.gov $
//==============================================================================
//         
//  Functor class.        
//         
//  Used by BrentMinimizer1D algotithm to find maxima in cross-section         
//  in W2 for a given t and Q2.         
//==============================================================================        
#ifndef ModeFinderFunctor_h         
#define ModeFinderFunctor_h         
#include "Math/IFunction.h"         
                  
class CrossSection;         
         
class ModeFinderFunctor : public ROOT::Math::IGenFunction {         
public:         
    ModeFinderFunctor();         
    ModeFinderFunctor(CrossSection*, double Q2, double vmMass, double tmin, double tmax);         
             
    double DoEval(double) const;         
    ROOT::Math::IGenFunction* Clone() const;          
    void setQ2(double);         
    void setVmMass(double);         
    void setMaxT(double);         
    void setMinT(double);         
             
private:         
    CrossSection *mCrossSection;         
    double mQ2;         
    double mVmMass;  
    double mMinT;  
    double mMaxT;    
};         
#endif         
         
