//==============================================================================
//  PhotonFlux.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2015-05-29 14:40:39 -0400 (Fri, 29 May 2015) $
//  $Author: tobilibob@gmail.com $
//==============================================================================
//         
//  Functor class.         
//  Photon flux is given in: d2sig/(dQ2 dW2)         
//==============================================================================       
#ifndef PhotonFlux_h         
#define PhotonFlux_h         
#include "Enumerations.h"         
#include "TH1D.h"
         
class Nucleus;

class PhotonFlux {         
public:         
    PhotonFlux();         
    PhotonFlux(double s);         
             
    void setS(double);         
             
    double operator()(double Q2, double W2, GammaPolarization p) const;         
    double nuclearPhotonFlux(double Q2, double W2) const;
             
private:         
    double fluxTransverse(double Q2, double W2) const;         
    double fluxLongitudinal(double Q2, double W2) const;         

    double nuclearFormFactor(double) const;
    double eGammaOfQ2andW2(double Q2, double W2) const;
    double derivativeOfEgamWithW2() const;

    double sigma_nn(double) const;
    double TAA(double);
    double TAAForIntegration(const double*) const;
    double bDependenceForIntegration(double*, double*);

    void calculateUPCImpactParameterFactor() ;
    double mUPCImpactParameterFactor;

    void calculateTAAlookupTable();

    Nucleus *mNucleus;
    Nucleus *mNucleusUPC;
    double mB;

    TH1D* mTAA_of_b;
         
    double mS;         
};         
#endif         
