//==============================================================================
//  TableCollection.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2013-05-29 16:26:19 -0400 (Wed, 29 May 2013) $
//  $Author: thomas.ullrich@bnl.gov $
//==============================================================================
#ifndef TableCollection_h         
#define TableCollection_h         
#include "Enumerations.h"         
#include <vector>         
#include <iostream>         
         
using namespace std;         
         
class Table;         
         
class TableCollection {         
public:         
    TableCollection();         
    TableCollection(const TableCollection&);         
    TableCollection(int A, DipoleModelType typ, int vmID);         
    ~TableCollection();         
   
    TableCollection& operator=(const TableCollection&);         
  
    bool init(int A, DipoleModelType typ, int vmID);         
    
    bool tableExists(GammaPolarization pol, AmplitudeMoment mom) const;         
    bool available(double Q2, double W2, double t, GammaPolarization p, AmplitudeMoment m) const;   
    
    double get(double Q2, double W2, double t, GammaPolarization p, AmplitudeMoment m) const;         
    double get(double Q2, double W2, double t, GammaPolarization p, AmplitudeMoment m, Table *&) const;         
         
    void list(ostream& = cout, bool = false) const;         
         
    double minQ2() const;           
    double maxQ2() const;         
    double minW2() const;         
    double maxW2() const;         
    double minW() const;         
    double maxW() const;         
    double minT() const;         
    double maxT() const;         
             
private:         
    double minimumValue(unsigned int) const;   
    double maximumValue(unsigned int) const;   
  
private:         
    vector<Table*> mTables;         
};         
#endif         
