//==============================================================================
//  TableGeneratorSettings.h
//
//  Copyright (C) 2010-2013 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre version: 1.1
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Tobias Toll
//  Last update: 
//  $Date: 2014-04-09 14:53:34 -0400 (Wed, 09 Apr 2014) $
//  $Author: tobilibob@gmail.com $
//==============================================================================
//  
//  Singleton class  
//  
//==============================================================================  
#ifndef TableGeneratorSettings_h         
#define TableGeneratorSettings_h         
#include "Settings.h"         
#include "Enumerations.h"  
         
using namespace std;         
         
class TableGeneratorSettings : public Settings {         
public:         
    static TableGeneratorSettings* instance();         
         
    void setVerbose(bool);         
    bool verbose() const;         
         
    void setVerboseLevel(int);         
    int verboseLevel() const;         
             
    void setQ2min(double);  
    double Q2min() const;  
      
    void setQ2max(double);  
    double Q2max() const;  
      
    void setWmin(double);  
    double Wmin() const;  
      
    void setWmax(double);  
    double Wmax() const;  
      
    void setTmin(double);  
    double tmin() const;  
      
    void setTmax(double);  
    double tmax() const;  
      
    void setQ2bins(unsigned int);  
    unsigned int Q2bins() const;  
      
    void setW2bins(unsigned int);  
    unsigned int W2bins() const;  
      
    void setTbins(unsigned int);  
    unsigned int tbins() const;  
      
    int vectorMesonId() const;  
    void setVectorMesonId(int);  
      
    DipoleModelType dipoleModel() const;  
    string dipoleModelName() const;  
    void setDipoleModel(DipoleModelType);  
      
    string rootfile() const;  
    void setRootfile(const char*);  
      
    unsigned int A() const;  
    void setA(unsigned int);  
      
    string bSatLookupPath() const;  
    void setBSatLookupPath(string);  
      
    unsigned int numberOfConfigurations() const;  
    void setNumberOfConfigurations(unsigned int);  
      
    bool useBackupFile() const;  
    void setUseBackupFile(bool);  
      
    int startingBinFromBackup() const;  
    void setStartingBinFromBackup(int);  
      
    int startBin() const;  
    void setStartBin(int);  
      
    int endBin() const;  
    void setEndBin(int);  
      
    int modesToCalculate() const;  
    void setModesToCalculate(int);  

    unsigned char priority() const;
    void setPriority(unsigned char);

    void consolidateSettings();    
      
private:         
    TableGeneratorSettings();         
             
private:         
    static TableGeneratorSettings* mInstance;         
             
private:         
    bool mVerbose;         
    int  mVerboseLevel;         
  
    unsigned int mSeed;  
    unsigned int mA;         
    unsigned int mNumberOfConfigurations;         
      
    bool mUseBackupFile;  
  
    int  mVectorMesonId;             
    int  mStartingBinFromBackup;  
    int  mStartBin, mEndBin;  
    int  mModesToCalculate;  
      
             
    double mQ2min;         
    double mQ2max;         
    double mWmin;         
    double mWmax;         
    double mTmin;         
    double mTmax;         
             
    unsigned int mQ2bins;         
    unsigned int mW2bins;         
    unsigned int mTbins;         
         
    DipoleModelType mDipoleModel;  
      
    string mRootfile;         
    string mBSatLookupPath;      
    string mDipoleModelName;  

    int mPriority;
};         
         
#endif         
